@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* test-main.c  main() for make check
  *
  * Copyright (C) 2003 Red Hat, Inc.
@@ -17,16 +17,18 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
 #include "test.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-sysdeps.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-message-private.h>
 #include "selinux.h"
 
 #ifdef DBUS_BUILD_TESTS
@@ -69,6 +71,7 @@ test_post_hook (void)
   if (_dbus_getenv ("DBUS_TEST_SELINUX"))
     bus_selinux_shutdown ();
   check_memleaks (progname);
+  _dbus_check_fdleaks();
 }
 
 int
@@ -76,6 +79,7 @@ main (int argc, char **argv)
 {
 #ifdef DBUS_BUILD_TESTS
   const char *dir;
+  const char *only;
   DBusString test_data_dir;
 
   progname = argv[0];
@@ -85,6 +89,11 @@ main (int argc, char **argv)
   else
     dir = _dbus_getenv ("DBUS_TEST_DATA");
 
+  if (argc > 2)
+    only = argv[2];
+  else
+    only = NULL;
+
   if (dir == NULL)
     {
       fprintf (stderr, "Must specify test data directory as argv[1] or in DBUS_TEST_DATA env variable\n");
@@ -93,53 +102,82 @@ main (int argc, char **argv)
 
   _dbus_string_init_const (&test_data_dir, dir);
 
-#if 0
-  /* FIXME this is disabled because of thread bugs that need fixing... */
   if (!_dbus_threads_init_debug ())
     die ("initializing debug threads");
+
+  if (only == NULL || strcmp (only, "expire-list") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running expire list test\n", argv[0]);
+      if (!bus_expire_list_test (&test_data_dir))
+        die ("expire list");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "config-parser") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running config file parser test\n", argv[0]);
+      if (!bus_config_parser_test (&test_data_dir))
+        die ("parser");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "policy") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running policy test\n", argv[0]);
+      if (!bus_policy_test (&test_data_dir))
+        die ("policy");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "signals") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running signals test\n", argv[0]);
+      if (!bus_signals_test (&test_data_dir))
+        die ("signals");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "dispatch-sha1") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running SHA1 connection test\n", argv[0]);
+      if (!bus_dispatch_sha1_test (&test_data_dir))
+        die ("sha1");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "dispatch") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running message dispatch test\n", argv[0]);
+      if (!bus_dispatch_test (&test_data_dir)) 
+        die ("dispatch");
+      test_post_hook ();
+    }
+
+  if (only == NULL || strcmp (only, "activation-service-reload") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running service files reloading test\n", argv[0]);
+      if (!bus_activation_service_reload_test (&test_data_dir))
+        die ("service reload");
+      test_post_hook ();
+    }
+
+#ifdef HAVE_UNIX_FD_PASSING
+  if (only == NULL || strcmp (only, "unix-fds-passing") == 0)
+    {
+      test_pre_hook ();
+      printf ("%s: Running unix fd passing test\n", argv[0]);
+      if (!bus_unix_fds_passing_test (&test_data_dir))
+        die ("unix fd passing");
+      test_post_hook ();
+    }
 #endif
- 
-  test_pre_hook ();
-  printf ("%s: Running expire list test\n", argv[0]);
-  if (!bus_expire_list_test (&test_data_dir))
-    die ("expire list");
-  test_post_hook ();
- 
-  test_pre_hook ();
-  printf ("%s: Running config file parser test\n", argv[0]);
-  if (!bus_config_parser_test (&test_data_dir))
-    die ("parser");
-  test_post_hook ();
-
-  test_pre_hook ();
-  printf ("%s: Running policy test\n", argv[0]);
-  if (!bus_policy_test (&test_data_dir))
-    die ("policy");
-  test_post_hook ();
-
-  test_pre_hook ();
-  printf ("%s: Running signals test\n", argv[0]);
-  if (!bus_signals_test (&test_data_dir))
-    die ("signals");
-  test_post_hook ();
-
-  test_pre_hook ();
-  printf ("%s: Running SHA1 connection test\n", argv[0]);
-  if (!bus_dispatch_sha1_test (&test_data_dir))
-    die ("sha1");
-  test_post_hook ();
-
-  test_pre_hook ();
-  printf ("%s: Running message dispatch test\n", argv[0]);
-  if (!bus_dispatch_test (&test_data_dir)) 
-    die ("dispatch");
-  test_post_hook ();
-
-  test_pre_hook ();
-  printf ("%s: Running service files reloading test\n", argv[0]);
-  if (!bus_activation_service_reload_test (&test_data_dir))
-    die ("service reload");
-  test_post_hook ();
 
   printf ("%s: Success\n", argv[0]);
 
