@@ -21,12 +21,14 @@
  *
  */
 
+#include <config.h>
 #include "dbus-mainloop.h"
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-sysdeps.h>
+#include <dbus/dbus-watch.h>
 
 #define MAINLOOP_SPEW 0
 
@@ -583,6 +585,7 @@ _dbus_loop_iterate (DBusLoop     *loop,
         {
           unsigned int flags;
           WatchCallback *wcb = WATCH_CALLBACK (cb);
+          int fd = dbus_watch_get_socket (wcb->watch);
 
           if (wcb->last_iteration_oom)
             {
@@ -598,9 +601,16 @@ _dbus_loop_iterate (DBusLoop     *loop,
 
 #if MAINLOOP_SPEW
               _dbus_verbose ("  skipping watch on fd %d as it was out of memory last time\n",
-                             dbus_watch_get_socket (wcb->watch));
+                             fd);
 #endif
             }
+          else if (_DBUS_UNLIKELY (fd == -1))
+            {
+              _dbus_warn ("watch %p was invalidated but not removed; "
+                  "removing it now\n", wcb->watch);
+              _dbus_loop_remove_watch (loop, wcb->watch, wcb->function,
+                  ((Callback *)wcb)->data);
+            }
           else if (dbus_watch_get_enabled (wcb->watch))
             {
               watches_for_fds[n_fds] = wcb;
@@ -609,7 +619,7 @@ _dbus_loop_iterate (DBusLoop     *loop,
                   
               flags = dbus_watch_get_flags (wcb->watch);
                   
-              fds[n_fds].fd = dbus_watch_get_socket (wcb->watch);
+              fds[n_fds].fd = fd;
               fds[n_fds].revents = 0;
               fds[n_fds].events = 0;
               if (flags & DBUS_WATCH_READABLE)
@@ -619,7 +629,7 @@ _dbus_loop_iterate (DBusLoop     *loop,
 
 #if MAINLOOP_SPEW
               _dbus_verbose ("  polling watch on fd %d  %s\n",
-                             fds[n_fds].fd, watch_flags_to_string (flags));
+                             fd, watch_flags_to_string (flags));
 #endif
 
               n_fds += 1;
@@ -628,7 +638,7 @@ _dbus_loop_iterate (DBusLoop     *loop,
             {
 #if MAINLOOP_SPEW
               _dbus_verbose ("  skipping disabled watch on fd %d  %s\n",
-                             dbus_watch_get_socket (wcb->watch),
+                             fd,
                              watch_flags_to_string (dbus_watch_get_flags (wcb->watch)));
 #endif
             }
@@ -819,6 +829,15 @@ _dbus_loop_iterate (DBusLoop     *loop,
                   
                   retval = TRUE;
                 }
+
+              if (_DBUS_UNLIKELY (fds[i].revents & _DBUS_POLLNVAL))
+                {
+                  _dbus_warn ("invalid request, socket fd %d not open\n",
+                      fds[i].fd);
+                  _dbus_watch_invalidate (wcb->watch);
+                  _dbus_loop_remove_watch (loop, wcb->watch, wcb->function,
+                      ((Callback *)wcb)->data);
+                }
             }
               
           ++i;
